package net.wanji.com.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.com.cache.SignalCommandInterfaceCache;
import net.wanji.com.common.constants.Constants;
import net.wanji.com.common.enums.SignalInterfaceTypeEnum;
import net.wanji.com.service.CommonService;
import net.wanji.com.service.controller.ControlCommandService;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.vo.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author duanruiming
 * @date 2023/05/08 10:45
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/controlCommand")
@Api(value = "信号指令服务")
public class ControlCommandController {

    private final SignalCommandInterfaceCache signalCommandInterfaceCache;
    private final CommonService commonService;

    /**
     * 通过厂商编码和类类型，获取缓存中实现类
     *
     * @param manufacturerId
     * @return
     */
    private ControlCommandService getService(String manufacturerId) {
        return  (ControlCommandService) (signalCommandInterfaceCache.getService(manufacturerId.concat(SignalInterfaceTypeEnum.CONTROL.getCode())));
    }

    /**
     * 信号机方案下发
     *
     * @param schemeSendVO 方案下发VO
     * @return
     */
    @AspectLog(description = "信号机方案下发-基础方案下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "信号机方案下发-基础方案下发", notes = "信号机方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public JsonViewObject schemeSend(@RequestBody @Validated SchemeSendVO schemeSendVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(schemeSendVO.getCrossCode());
        return getService(manufacturerId).schemeSend(schemeSendVO);
    }

    /**
     * 下发信号机计划
     *
     * @param planSendVO 计划下发VO
     * @return
     */
    @AspectLog(description = "计划下发-计划信息、时段信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public JsonViewObject planSend(@RequestBody @Validated PlanSendVO planSendVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(planSendVO.getCrossCode());
        return getService(manufacturerId).planSend(planSendVO);
    }

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "时间表下发-时间表信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public JsonViewObject scheduleSend(@RequestBody @Validated ScheduleSendVO scheduleSendVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(scheduleSendVO.getCrossCode());
        return getService(manufacturerId).scheduleSend(scheduleSendVO);
    }

    /**
     * 相位配时下发-（下发相位参数）
     *
     * @return
     * @throws InterruptedException 中断异常
     */
    @AspectLog(description = "相位配时下发-（下发相位参数）", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "相位配时下发-（下发相位参数）", notes = "相位配时下发-（下发相位参数）")
    @PostMapping("/phaseTimingSend")
    public JsonViewObject phaseTimingSend(@RequestBody @Validated PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(phaseTimingSendVO.getCrossCode());
        return getService(manufacturerId).phaseTimingSend(phaseTimingSendVO);
    }

    /**
     * 全红控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "全红控制-路口全红控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "全红控制-路口全红控制/恢复", notes = "全红控制-路口全红控制/恢复")
    @PostMapping("/allRedControl")
    public JsonViewObject allRedControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        return getService(manufacturerId).setSignalControl(commandVO.getCrossCode(), commandVO.getCommand(), Constants.ALL_RED_CONTROL);
    }

    /**
     * 黄闪控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "黄闪控制-路口黄闪控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "黄闪控制-路口黄闪控制/恢复", notes = "黄闪控制-路口黄闪控制/恢复")
    @PostMapping("/yellowLightControl")
    public JsonViewObject yellowLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        return getService(manufacturerId).setSignalControl(commandVO.getCrossCode(), commandVO.getCommand(), Constants.YELLOW_LIGHT_CONTROL);
    }

    /**
     * 关灯控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "关灯控制-路口关灯控制/开灯", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "关灯控制-路口关灯控制/开灯", notes = "关灯控制-路口关灯控制/开灯")
    @PostMapping("/closeLightControl")
    public JsonViewObject closeLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        return getService(manufacturerId).setSignalControl(commandVO.getCrossCode(), commandVO.getCommand(), Constants.CLOSE_LIGHT_CONTROL);
    }

    /**
     * 步进控制
     *
     * @param crossCode 信号id
     * @param command   命令
     * @param stepNum   一步num
     * @return
     */
    @AspectLog(description = "步进控制-步进控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossCode", value = "13位路口编号", paramType = "query", required = true, dataType = "String"),
            @ApiImplicitParam(name = "command", value = "1 开始步进 0 取消步进", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = "stepNum", value = "0 顺序步进 n 跳过n个相位", paramType = "query", required = true, dataType = "int"),
    })
    @PostMapping("/stepControl")
    public JsonViewObject stepControl(@RequestParam String crossCode,
                                      @RequestParam Integer command,
                                      @RequestParam int stepNum) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(crossCode);
        return getService(manufacturerId).stepControl(crossCode, command, stepNum);
    }

    /**
     * 恢复时间表
     *
     * @param crossCode
     * @return
     */
    @AspectLog(description = "恢复时间表-恢复路口时间表执行", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/recoverSchedule")
    public JsonViewObject recoverSchedule(@RequestParam String crossCode) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(crossCode);
        return getService(manufacturerId).recoverSchedule(crossCode);
    }

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "锁定控制-相位锁定/解锁", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public JsonViewObject lockControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        return getService(manufacturerId).lockControl(commandVO);
    }

    /**
     * 相位差下发
     */
    @AspectLog(description = "相位差下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "相位差下发", notes = "相位差下发")
    @PostMapping("/phaseDiffSend")
    public JsonViewObject phaseDiffSend(String crossCode) {
        String manufacturerId = commonService.getManufacturerCodeByCrossId(crossCode);
        return getService(manufacturerId).phaseDiffSend();
    }

/*    @AspectLog(description = "删除信号机配置参数", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "删除信号机配置参数", notes = "删除信号机配置参数")
    @PostMapping("/delBaseConfig")
    public JsonViewObject delBaseConfig(@RequestBody @Validated DelBaseConfigVO delBaseConfigVO) throws Exception {
        return signalTypeServiceImplCache.getService(manufacturerId).delBaseConfig(delBaseConfigVO);
    }*/
}
